function opt = get_def_opt()
% Usage: function opt = get_def_opt()
%
% Initializes the options struct with the default settings
% Some variables such as opt.nAF, opt.mu and opt.nUpdates are highly 
% recommended to be adjusted manually as they vary greatly depending on 
% the problem in question.
%



%% MP settings
opt.nAF = 3;                % number of atomic functions
opt.nUpdates = 2000;         % learning updates of the atomic functions (only of the ones selected by MP, of course)
                            % equals to the total number of mp-decompositions
opt.mu = 0.001;             % gradient descent learning rate
opt.threshi_frac = 4;       % atomsize-change parameter (high values (>4) facilitate growth and shrinkage of the atomic functions)
                            % is only used if opt.vary_atomsize = 1;
                            
% Keep these settings as they are
opt.vary_atomsize = 1;      % adapt length of atomic functions?
opt.normalizeAFs = 1;       % normalize atomic function after each update?
opt.vary_AFsize = 1;        % start with atomic functions of different length
opt.AFsizes = [0.0625, 0.125, 0.25,  0.5, 1; ...  % AF length (ratio of DICTLEN)
               0, 1, 0, 0, 0];                    % fraction or total number of AFs with this length


%% MP settings
opt.mp.stop = 'corr';       % stoppping criterion: 'SNR', 'atoms' or 'corr'
opt.mp.SNR = 3;             % "signal-to-noise" ratio (only used when opt.mp.stop = 'SNR')
opt.mp.corrthresh = 0.2;    % "spiking threshold" (only used when opt.mp.stop = 'corr')
opt.mp.mp_version = 'mp';  % 'mp' or 'omp' (matching pursuit or orthogonal matching pursuit)
opt.mp.normalize_dict = 1;  % normalize dictionary atomic functions to l2-norm of 1?


% Keep these settings as they are
opt.mp.verbose =  0;
opt.mp.reuse_atom =  1;     % atom can be used more than once per decomposition?               
opt.mp.snr_every = 1;       % compute and store snr every X iterations
