%    Sparse Coding Classification Code -- (c) 2009-2011 Simon Scholler,
%    Music Technology Group, UPF, Barcelona,
%    and Bernstein Center of Computational Neuroscience, Berlin
%
%    This code is provided as it is, with no guarantees.  Published reports of research
%    using this code (or a modified version, maintaining a significant portion of the original code)
%    should cite the following article:
%
%    Scholler, S., Purwins, H., Sparse Approximations for Drum Sound Classification,
%    IEEE Journal of Selected Topics in Signal Processing, Vol. 5, No. 5:pp. 933-940, 2011 
%
%    Comments and bug reports are welcome.  Email to
%    simon.scholler@googlemail.com.
%    I would also appreciate hearing about how you used this code,
%
%    You are free to use this code and
%    to modify, extend or distribute this code, as long
%    as this copyright notice is included whole and unchanged.
%

function [basis sizechange] = update_atomsize(basis,book_idx,thresh_i, thresh_d)
   global PL DICTLEN THRESHI_FRAC
   if isempty(PL)
       PL = 10;  % padlength (in percent)
   end
   if nargin<2
       book_idx = 1:length(basis);
   end
   if nargin<3
       bf_10percent_dotpr = 0.1; %1/(10+(2*PL/10));
       if isempty(THRESHI_FRAC)
           error('empty')
       else
       thresh_i = (PL/10)*bf_10percent_dotpr/THRESHI_FRAC;    % increase atomsize threshold
       end
       thresh_d = thresh_i/2;                                 % decrease atomsize threshold
   end
      
   sizechange = zeros(1,length(book_idx));
   for n = 1:length(book_idx)                    
      bi = book_idx(n);      
      if abs(l2norm(basis{bi})-1)>0.001
          error('basis function is not normalized.')
      end
      
      %% INCREASE AF-SIZE
      [bf lpad rpad] = unpad_bf(basis{bi});    
      li = dot(lpad,lpad)>thresh_i;
      ri = dot(rpad,rpad)>thresh_i;        
      if li && ri               
          sizechange(n) = sizechange(n)+2;
          bf2 = [lpad; bf; rpad];          
          basis{bi} = pad_bf(bf2);                    
      elseif li
          sizechange(n) = sizechange(n)+1;
          bf2 = [lpad; bf];
          basis{bi} = pad_bf(bf2,[],rpad);          
      elseif ri          
          sizechange(n) = sizechange(n)+1;
          bf2 = [bf; rpad];
          basis{bi} = pad_bf(bf2,lpad,[]);
      end
      
      %% DECREASE AF-SIZE
      [bf3 lpad2 rpad2] = unpad_bf(basis{bi}); 
      l = length(lpad); 
      lend = bf3(1:l);
      rend = bf3(end-l+1:end);
      ld = dot(lend,lend)<thresh_d;
      rd = dot(rend,rend)<thresh_d;  
      minlen = 20;
      if ld
          sizechange(n) = sizechange(n)+1;
          l = min(length(bf3)-minlen,l)+1;
          bf4 = bf3(l:end);
          basis{bi} = pad_bf(bf4,bf3(1:l-1),[]);      
      end
      if rd
          sizechange(n) = sizechange(n)+1;
          r = max(minlen,length(bf3)-l);
          bf4 = bf3(1:r);
          basis{bi} = pad_bf(bf4,[],bf3(r+1:end));      
      end
      if ld || rd
          basis{bi} = pad_bf(bf4);      
      end
   end   
end


%% SUBFUNCTIONS
function [bf,padl,padr] = unpad_bf(bf)
global PL
pl = round((PL/10)*length(bf)/(10+2*PL/10));
if nargout>1
    padl = bf(1:pl);
    padr = bf(end-pl+1:end);
end
bf = bf(pl+1:end-pl);
end

function bf = pad_bf(bf,padl,padr)
global PL
pl = round(length(bf)*(PL/100));
global DICTLEN
if length(bf)+pl*2>DICTLEN
    pl = floor((DICTLEN-length(bf))/2);
end
if nargin>1
    if pl-length(padl)>=0
        padl = [zeros(pl-length(padl),1); padl];
    else
        padl = padl(end-pl+1:end);
    end
    if pl-length(padr)>=0
        padr = [padr; zeros(pl-length(padr),1)];
    else
        padr = padr(1:pl); 
    end
    bf = [padl; bf; padr];
else
    bf = [zeros(pl,1); bf; zeros(pl,1)];
end
end


   